@@ -3,6 +3,14 @@
 import torch
 
 
+def choice(x, a):
+    """Generate a random sample from an array of given size."""
+    if torch.is_tensor(x):
+        return x[torch.randint(len(x), (a,))]
+
+    return x
+
+
 def rand(*args, **kwargs):
     return torch.rand(*args, **kwargs)
 
