@@ -3,6 +3,15 @@
 import tensorflow as tf
 
 
+def choice(x, size, axis=0):
+    dim_x = tf.cast(tf.shape(x)[axis], tf.int64)
+    indices = tf.range(0, dim_x, dtype=tf.int64)
+    sample_index = tf.random.shuffle(indices)[:size]
+    sample = tf.gather(x, sample_index, axis=axis)
+
+    return sample
+
+
 def randint(low, high=None, size=None):
     if size is None:
         size = (1,)
