@@ -5,7 +5,6 @@ The n-dimensional hyperbolic space embedded and its different representations.
 
 
 import geomstats.backend as gs
-import geomstats.vectorization
 from geomstats.geometry.manifold import Manifold
 from geomstats.geometry.riemannian_metric import RiemannianMetric
 
@@ -40,8 +39,8 @@ class Hyperbolic(Manifold):
     default_coords_type = 'extrinsic'
     default_point_type = 'vector'
 
-    def __init__(self, dim, scale=1):
-        super(Hyperbolic, self).__init__(dim=dim)
+    def __init__(self, dim, scale=1, **kwargs):
+        super(Hyperbolic, self).__init__(dim=dim, **kwargs)
         self.point_type = Hyperbolic.default_point_type
         self.coords_type = Hyperbolic.default_coords_type
         self.scale = scale
@@ -51,25 +50,22 @@ class Hyperbolic(Manifold):
     def _extrinsic_to_extrinsic_coordinates(point):
         """Convert the parameterization of a point.
 
-        Convert the parameterization of a point in hyperbolic space
-        from its extrinsic coordinates, to its intrinsic coordinates
-        in Minkowski space.
+        Dummy method that returns the input point.
 
         Parameters
         ----------
-        point_extrinsic : array-like, shape=[..., dim + 1]
+        point : array-like, shape=[..., dim + 1]
             Point in hyperbolic space in extrinsic coordinates.
 
         Returns
         -------
-        point_intrinsic : array-like, shape=[..., dim]
-            Point in hyperbolic space in intrinsic coordinates.
+        point_extrinsic : array-like, shape=[..., dim]
+            Point in hyperbolic space in extrinsic coordinates.
         """
         return point
 
     @staticmethod
-    @geomstats.vectorization.decorator(['vector'])
-    def _intrinsic_to_extrinsic_coordinates(point_intrinsic):
+    def _intrinsic_to_extrinsic_coordinates(point):
         """Convert intrinsic to extrinsic coordinates.
 
         Convert the parameterization of a point in hyperbolic space
@@ -78,7 +74,7 @@ class Hyperbolic(Manifold):
 
         Parameters
         ----------
-        point_intrinsic : array-like, shape=[..., dim]
+        point : array-like, shape=[..., dim]
             Point in hyperbolic space in intrinsic coordinates.
 
         Returns
@@ -86,16 +82,14 @@ class Hyperbolic(Manifold):
         point_extrinsic : array-like, shape=[..., dim + 1]
             Point in hyperbolic space in extrinsic coordinates.
         """
-        coord_0 = gs.sqrt(1. + gs.linalg.norm(point_intrinsic, axis=-1) ** 2)
-        coord_0 = gs.to_ndarray(coord_0, to_ndim=1)
-        coord_0 = gs.to_ndarray(coord_0, to_ndim=2, axis=1)
-
-        point_extrinsic = gs.hstack([coord_0, point_intrinsic])
+        coord_0 = gs.sqrt(1. + gs.sum(point ** 2, axis=-1))
+        point_extrinsic = gs.concatenate(
+            [coord_0[..., None], point], axis=-1)
 
         return point_extrinsic
 
     @staticmethod
-    def _extrinsic_to_intrinsic_coordinates(point_extrinsic):
+    def _extrinsic_to_intrinsic_coordinates(point):
         """Convert extrinsic to intrinsic coordinates.
 
         Convert the parameterization of a point in hyperbolic space
@@ -104,14 +98,14 @@ class Hyperbolic(Manifold):
 
         Parameters
         ----------
-        point_extrinsic : array-like, shape=[..., dim + 1]
+        point : array-like, shape=[..., dim + 1]
             Point in hyperbolic space in extrinsic coordinates.
 
         Returns
         -------
         point_intrinsic : array-like, shape=[..., dim]
         """
-        point_intrinsic = point_extrinsic[..., 1:]
+        point_intrinsic = point[..., 1:]
 
         return point_intrinsic
 
@@ -120,7 +114,7 @@ class Hyperbolic(Manifold):
         """Convert extrinsic to ball coordinates.
 
         Convert the parameterization of a point in hyperbolic space
-        from its intrinsic coordinates, to the poincare ball model
+        from its intrinsic coordinates, to the Poincare ball model
         coordinates.
 
         Parameters
@@ -133,15 +127,16 @@ class Hyperbolic(Manifold):
         point_ball : array-like, shape=[..., dim]
             Point in hyperbolic space in Poincare ball coordinates.
         """
-        return point[..., 1:] / (1 + point[..., :1])
+        point_ball = point[..., 1:] / (1 + point[..., :1])
+
+        return point_ball
 
     @staticmethod
-    @geomstats.vectorization.decorator(['vector'])
     def _ball_to_extrinsic_coordinates(point):
         """Convert ball to extrinsic coordinates.
 
         Convert the parameterization of a point in hyperbolic space
-        from its poincare ball model coordinates, to the extrinsic
+        from its Poincare ball model coordinates, to the extrinsic
         coordinates.
 
         Parameters
@@ -151,86 +146,192 @@ class Hyperbolic(Manifold):
 
         Returns
         -------
-        extrinsic : array-like, shape=[..., dim + 1]
+        point_extrinsic : array-like, shape=[..., dim + 1]
             Point in hyperbolic space in extrinsic coordinates.
         """
         squared_norm = gs.sum(point**2, -1)
         denominator = 1 - squared_norm
-        t = gs.to_ndarray((1 + squared_norm) / denominator, to_ndim=2, axis=1)
-        expanded_denominator = gs.expand_dims(denominator, -1)
-        expanded_denominator = gs.tile(
-            expanded_denominator, (1, point.shape[-1]))
-        intrinsic = (2 * point) / expanded_denominator
-        return gs.concatenate([t, intrinsic], -1)
+        t = (1 + squared_norm) / denominator
+        intrinsic = gs.einsum(
+            '...i, ...->...i', 2 * point, 1. / denominator)
+        point_extrinsic = gs.concatenate([t[..., None], intrinsic], -1)
+        return point_extrinsic
 
-    @staticmethod
-    @geomstats.vectorization.decorator(['vector'])
-    def _half_plane_to_extrinsic_coordinates(point):
-        """Convert half plane to extrinsic coordinates.
+    @classmethod
+    def _half_space_to_extrinsic_coordinates(cls, point):
+        """Convert half-space to extrinsic coordinates.
 
-        Convert the parameterization of a point in the hyperbolic plane
-        from its upper half plane model coordinates, to the extrinsic
+        Convert the parameterization of a point in the hyperbolic space
+        from its Poincare half-space coordinates, to the extrinsic
         coordinates.
 
         Parameters
         ----------
-        point : array-like, shape=[..., 2]
-            Point in hyperbolic space in half-plane coordinates.
+        point : array-like, shape=[..., dim]
+            Point in hyperbolic space in half-space coordinates.
 
         Returns
         -------
-        extrinsic : array-like, shape=[..., 3]
-            Point in hyperbolic plane in extrinsic coordinates.
+        point_extrinsic : array-like, shape=[..., dim + 1]
+            Point in hyperbolic space in extrinsic coordinates.
         """
-        x, y = point[:, 0], point[:, 1]
-        x2 = point[:, 0]**2
-        den = x2 + (1 + y)**2
-        x = gs.to_ndarray(x, to_ndim=2, axis=0)
-        y = gs.to_ndarray(y, to_ndim=2, axis=0)
-        x2 = gs.to_ndarray(x2, to_ndim=2, axis=0)
-        den = gs.to_ndarray(den, to_ndim=2, axis=0)
-        ball_point = gs.hstack((2 * x / den, (x2 + y**2 - 1) / den))
-        return Hyperbolic._ball_to_extrinsic_coordinates(ball_point)
+        point_ball = cls.half_space_to_ball_coordinates(point)
+        point_extrinsic = cls._ball_to_extrinsic_coordinates(point_ball)
 
-    @staticmethod
-    @geomstats.vectorization.decorator(['vector'])
-    def _extrinsic_to_half_plane_coordinates(point):
-        """Convert extrinsic to half plane coordinates.
+        return point_extrinsic
 
-        Convert the parameterization of a point in the hyperbolic plane
-        from its intrinsic coordinates, to the poincare upper half plane
+    @classmethod
+    def _extrinsic_to_half_space_coordinates(cls, point):
+        """Convert extrinsic to half-space coordinates.
+
+        Convert the parameterization of a point in the hyperbolic space
+        from its extrinsic coordinates, to the Poincare half-space
         coordinates.
 
         Parameters
         ----------
-        point : array-like, shape=[..., 2]
-            Point in the hyperbolic plane in intrinsic coordinates.
+        point : array-like, shape=[..., dim + 1]
+            Point in the hyperbolic space in extrinsic coordinates.
 
         Returns
         -------
-        point_half_plane : array-like, shape=[..., 2]
-            Point in the hyperbolic plane in Poincare upper half-plane
-            coordinates.
+        point_half_space : array-like, shape=[..., dim]
+            Point in the hyperbolic space in half-space coordinates.
         """
         point_ball = \
-            Hyperbolic._extrinsic_to_ball_coordinates(point)
-        point_ball_x, point_ball_y = point_ball[:, 0], point_ball[:, 1]
-        point_ball_x2 = point_ball_x**2
-        denom = point_ball_x2 + (1 - point_ball_y)**2
+            cls._extrinsic_to_ball_coordinates(point)
+        point_half_space = cls.ball_to_half_space_coordinates(point_ball)
 
-        point_ball_x = gs.to_ndarray(
-            point_ball_x, to_ndim=2, axis=0)
-        point_ball_y = gs.to_ndarray(
-            point_ball_y, to_ndim=2, axis=0)
-        point_ball_x2 = gs.to_ndarray(
-            point_ball_x2, to_ndim=2, axis=0)
-        denom = gs.to_ndarray(
-            denom, to_ndim=2, axis=0)
+        return point_half_space
 
-        point_half_plane = gs.hstack((
-            (2 * point_ball_x) / denom,
-            (1 - point_ball_x2 - point_ball_y**2) / denom))
-        return point_half_plane
+    @staticmethod
+    def half_space_to_ball_coordinates(point):
+        """Convert half-space to ball coordinates.
+
+        Convert the parameterization of a point in the hyperbolic space
+        from its Poincare half-space coordinates, to the Poincare ball
+        coordinates.
+
+        Parameters
+        ----------
+        point : array-like, shape=[..., dim]
+            Point in the hyperbolic space in half-space coordinates.
+
+        Returns
+        -------
+        point_ball : array-like, shape=[..., dim]
+            Point in the hyperbolic space in Poincare ball coordinates.
+        """
+        sq_norm = gs.sum(point ** 2, axis=-1)
+        den = 1 + sq_norm + 2 * point[..., -1]
+        component_1 = gs.einsum(
+            '...i,...->...i', point[..., :-1], 2. / den)
+        component_2 = (sq_norm - 1) / den
+        point_ball = gs.concatenate(
+            [component_1, component_2[..., None]], axis=-1)
+
+        return point_ball
+
+    @staticmethod
+    def ball_to_half_space_coordinates(point):
+        """Convert ball to half space coordinates.
+
+        Convert the parameterization of a point in the hyperbolic space
+        from its Poincare ball coordinates, to the Poincare half-space
+        coordinates.
+
+        Parameters
+        ----------
+        point : array-like, shape=[..., dim]
+            Point in the hyperbolic space in Poincare ball coordinates.
+
+        Returns
+        -------
+        point_ball : array-like, shape=[..., dim]
+            Point in the hyperbolic space in half-space coordinates.
+        """
+        sq_norm = gs.sum(point ** 2, axis=-1)
+        den = 1 + sq_norm - 2 * point[..., -1]
+        component_1 = gs.einsum(
+            '...i,...->...i', point[..., :-1], 2. / den)
+        component_2 = (1 - sq_norm) / den
+        point_half_space = gs.concatenate(
+            [component_1, component_2[..., None]], axis=-1)
+
+        return point_half_space
+
+    @staticmethod
+    def half_space_to_ball_tangent(tangent_vec, base_point):
+        """Convert half-space to ball tangent coordinates.
+
+        Convert the parameterization of a tangent vector to the
+        hyperbolic space from its Poincare half-space coordinates, to
+        the Poinare ball coordinates.
+
+        Parameters
+        ----------
+        tangent_vec : array-like, shape=[..., dim]
+            Tangent vector at the base point in the Poincare half-space.
+        base_point : array-like, shape=[..., dim]
+            Point in the Poincare half-space.
+
+        Returns
+        -------
+        tangent_vec_ball : array-like, shape=[..., dim]
+           Tangent vector in the Poincare ball.
+        """
+        sq_norm = gs.sum(base_point ** 2, axis=-1)
+        den = 1. + sq_norm + 2. * base_point[..., -1]
+        scalar_prod = gs.sum(base_point * tangent_vec, axis=-1)
+        component_1 = (
+            gs.einsum('...i,...->...i', tangent_vec[..., :-1], 2. / den)
+            - 4. * gs.einsum(
+                '...i,...->...i', base_point[..., :-1],
+                (scalar_prod + tangent_vec[..., -1]) / den**2))
+        component_2 = 2 * scalar_prod / den \
+            - 2 * (sq_norm - 1) * (scalar_prod + tangent_vec[..., -1]) \
+            / den ** 2
+        tangent_vec_ball = gs.concatenate(
+            [component_1, component_2[..., None]], axis=-1)
+
+        return tangent_vec_ball
+
+    @staticmethod
+    def ball_to_half_space_tangent(tangent_vec, base_point):
+        """Convert ball to half-space tangent coordinates.
+
+        Convert the parameterization of a tangent vector to the
+        hyperbolic space from its Poincare ball coordinates, to
+        the Poinare half-space coordinates.
+
+        Parameters
+        ----------
+        tangent_vec : array-like, shape=[..., dim]
+            Tangent vector at the base point in the Poincare ball.
+        base_point : array-like, shape=[..., dim]
+            Point in the Poincare ball.
+
+        Returns
+        -------
+        tangent_vec_half_spacel : array-like, shape=[..., dim]
+            Tangent vector in the Poincare half-space.
+
+        """
+        sq_norm = gs.sum(base_point ** 2, axis=-1)
+        den = 1 + sq_norm - 2 * base_point[..., -1]
+        scalar_prod = gs.sum(base_point * tangent_vec, -1)
+        component_1 = (
+            gs.einsum('...i,...->...i', tangent_vec[..., :-1], 2. / den)
+            - 4. * gs.einsum(
+                '...i,...->...i', base_point[..., :-1],
+                (scalar_prod - tangent_vec[..., -1]) / den**2))
+        component_2 = -2. * scalar_prod / den \
+            - 2 * (1. - sq_norm) * (scalar_prod - tangent_vec[..., -1]) \
+            / den**2
+        tangent_vec_half_space = gs.concatenate(
+            [component_1, component_2[..., None]], axis=-1)
+
+        return tangent_vec_half_space
 
     @staticmethod
     def change_coordinates_system(point,
@@ -270,10 +371,10 @@ class Hyperbolic(Manifold):
                 Hyperbolic._intrinsic_to_extrinsic_coordinates,
             'extrinsic-intrinsic':
                 Hyperbolic._extrinsic_to_intrinsic_coordinates,
-            'extrinsic-half-plane':
-                Hyperbolic._extrinsic_to_half_plane_coordinates,
-            'half-plane-extrinsic':
-                Hyperbolic._half_plane_to_extrinsic_coordinates,
+            'extrinsic-half-space':
+                Hyperbolic._extrinsic_to_half_space_coordinates,
+            'half-space-extrinsic':
+                Hyperbolic._half_space_to_extrinsic_coordinates,
             'extrinsic-extrinsic':
                 Hyperbolic._extrinsic_to_extrinsic_coordinates
         }
@@ -321,7 +422,7 @@ class Hyperbolic(Manifold):
         ----------
         point : array-like, shape=[..., {dim, dim + 1}]
             Point in hyperbolic space.
-        to_point_type : str, {'extrinsic', 'intrinsic', etc}
+        to_coords_type : str, {'extrinsic', 'intrinsic', etc}
             Coordinates type.
             Optional, default: 'ball'.
 
@@ -344,7 +445,7 @@ class Hyperbolic(Manifold):
         ----------
         point : array-like, shape=[..., {dim, dim + 1}]
             Point in hyperbolic space in coordinates from_point_type.
-        from_point_type : str, {'ball', 'extrinsic', 'intrinsic', 'half_plane'}
+        from_coords_type : str, {'ball', 'extrinsic', 'intrinsic', ...}
             Coordinates type.
 
         Returns
