@@ -6,25 +6,28 @@ import matplotlib.pyplot as plt
 import geomstats.backend as gs
 from geomstats.geometry.hyperboloid import Hyperboloid
 from geomstats.geometry.hypersphere import Hypersphere
+from geomstats.geometry.poincare_half_space import PoincareHalfSpace
 from geomstats.geometry.special_euclidean import SpecialEuclidean
 from geomstats.geometry.special_orthogonal import SpecialOrthogonal
 from mpl_toolkits.mplot3d import Axes3D  # NOQA
 
 SE3_GROUP = SpecialEuclidean(n=3, point_type='vector')
+SE2_GROUP = SpecialEuclidean(n=2, point_type='matrix')
+SE2_VECT = SpecialEuclidean(n=2, point_type='vector')
 SO3_GROUP = SpecialOrthogonal(n=3, point_type='vector')
 S1 = Hypersphere(dim=1)
 S2 = Hypersphere(dim=2)
 H2 = Hyperboloid(dim=2)
+POINCARE_HALF_PLANE = PoincareHalfSpace(dim=2)
 
 AX_SCALE = 1.2
 
-IMPLEMENTED = ['SO3_GROUP', 'SE3_GROUP', 'S1', 'S2',
+IMPLEMENTED = ['SO3_GROUP', 'SE3_GROUP', 'SE2_GROUP', 'S1', 'S2',
                'H2_poincare_disk', 'H2_poincare_half_plane', 'H2_klein_disk',
                'poincare_polydisk']
 
 
 def tutorial_matplotlib():
-    plt.style.use('seaborn')
     fontsize = 12
     matplotlib.rc('font', size=fontsize)
     matplotlib.rc('text')
@@ -37,7 +40,7 @@ def tutorial_matplotlib():
         monospace=['Computer Modern Typewriter'])
 
 
-class Arrow3D():
+class Arrow3D:
     """An arrow in 3d, i.e. a point and a vector."""
 
     def __init__(self, point, vector):
@@ -51,7 +54,7 @@ class Arrow3D():
                   **quiver_kwargs)
 
 
-class Trihedron():
+class Trihedron:
     """A trihedron, i.e. 3 Arrow3Ds at the same point."""
 
     def __init__(self, point, vec_1, vec_2, vec_3):
@@ -78,7 +81,7 @@ class Trihedron():
             self.arrow_3.draw(ax, color=green, **arrow_draw_kwargs)
 
 
-class Circle():
+class Circle:
     """Class used to draw a circle."""
 
     def __init__(self, n_angles=100, points=None):
@@ -120,7 +123,7 @@ class Circle():
                 **plot_kwargs)
 
 
-class Sphere():
+class Sphere:
     """Create the arrays sphere_x, sphere_y, sphere_z to plot a sphere.
 
     Create the arrays sphere_x, sphere_y, sphere_z of values
@@ -235,12 +238,11 @@ class Sphere():
                    cmap=plt.get_cmap(cmap))
 
 
-class PoincareDisk():
+class PoincareDisk:
     def __init__(self, points=None, point_type='extrinsic'):
         self.center = gs.array([0., 0.])
         self.points = []
         self.point_type = point_type
-
         if points is not None:
             self.add_points(points)
 
@@ -290,7 +292,7 @@ class PoincareDisk():
                 raise ValueError('Points do not have dimension 2.')
 
 
-class PoincarePolyDisk():
+class PoincarePolyDisk:
     """Class used to plot points in the Poincare polydisk."""
 
     def __init__(self, points=None, point_type='ball', n_disks=2):
@@ -340,32 +342,37 @@ class PoincarePolyDisk():
         ax.scatter(points_x, points_y, **kwargs)
 
 
-class PoincareHalfPlane():
+class PoincareHalfPlane:
     """Class used to plot points in the Poincare Half Plane."""
 
-    def __init__(self, points=None):
+    def __init__(self, points=None, point_type='half-space'):
         self.points = []
+        self.point_type = point_type
         if points is not None:
             self.add_points(points)
 
     def add_points(self, points):
-        if not gs.all(H2.belongs(points)):
-            raise ValueError(
-                'Points do not belong to the hyperbolic space.')
-        points = self.convert_to_half_plane_coordinates(points)
+        if self.point_type == 'extrinsic':
+            if not gs.all(H2.belongs(points)):
+                raise ValueError(
+                    'Points do not belong to the hyperbolic space '
+                    '(extrinsic coordinates)')
+            points = self.convert_to_half_plane_coordinates(points)
+        elif self.point_type == 'half-space':
+            if not gs.all(POINCARE_HALF_PLANE.belongs(points)):
+                raise ValueError(
+                    'Points do not belong to the hyperbolic space '
+                    '(Poincare half plane coordinates).')
         if not isinstance(points, list):
             points = list(points)
         self.points.extend(points)
 
-    @staticmethod
-    def set_ax(ax=None):
+    def set_ax(self, points, ax=None):
         if ax is None:
             ax = plt.subplot()
-        ax_s = AX_SCALE
-        plt.setp(ax,
-                 xlim=(-ax_s, ax_s),
-                 ylim=(0., ax_s),
-                 xlabel='X', ylabel='Y')
+        if self.point_type == 'extrinsic':
+            points = self.convert_to_half_plane_coordinates(points)
+        plt.setp(ax, xlabel='X', ylabel='Y')
         return ax
 
     @staticmethod
@@ -389,7 +396,7 @@ class PoincareHalfPlane():
         ax.scatter(points_x, points_y, **kwargs)
 
 
-class KleinDisk():
+class KleinDisk:
     def __init__(self, points=None):
         self.center = gs.array([0., 0.])
         self.points = []
@@ -442,6 +449,47 @@ class KleinDisk():
         ax.scatter(points_x, points_y, **kwargs)
 
 
+class SpecialEuclidean2:
+    """Class used to plot points in the 2d special euclidean group."""
+
+    def __init__(self, points=None, point_type='matrix'):
+        self.points = []
+        self.point_type = point_type
+        if points is not None:
+            self.add_points(points)
+
+    @staticmethod
+    def set_ax(ax=None, x_lim=None, y_lim=None):
+        if ax is None:
+            ax = plt.subplot()
+        if x_lim is not None:
+            ax.set_xlim(x_lim)
+        if y_lim is not None:
+            ax.set_ylim(y_lim)
+        return ax
+
+    def add_points(self, points):
+        if self.point_type == 'vector':
+            points = SE2_VECT.matrix_from_vector(points)
+        if not gs.all(SE2_GROUP.belongs(points)):
+            raise ValueError(
+                'Points do not belong to SE2.')
+        if not isinstance(points, list):
+            points = list(points)
+        self.points.extend(points)
+
+    def draw(self, ax, **kwargs):
+        points = gs.array(self.points)
+        translation = points[..., :2, 2]
+        frame_1 = points[:, :2, 0]
+        frame_2 = points[:, :2, 1]
+        ax.quiver(translation[:, 0], translation[:, 1], frame_1[:, 0],
+                  frame_1[:, 1], width=0.005, color='b')
+        ax.quiver(translation[:, 0], translation[:, 1], frame_2[:, 0],
+                  frame_2[:, 1], width=0.005, color='r')
+        ax.scatter(translation[:, 0], translation[:, 1], s=16, **kwargs)
+
+
 def convert_to_trihedron(point, space=None):
     """Transform a rigid point into a trihedron.
 
@@ -487,11 +535,25 @@ def convert_to_trihedron(point, space=None):
 
 
 def plot(points, ax=None, space=None,
-         point_type='extrinsic', **point_draw_kwargs):
-    """Plot points in the 3D Special Euclidean Group.
+         point_type=None, **point_draw_kwargs):
+    """Plot points in one of the implemented manifolds.
 
-    Plot points in the 3D Special Euclidean Group,
-    by showing them as trihedrons.
+    The implemented manifolds are:
+    - the special orthogonal group SO(3)
+    - the special Euclidean group SE(3)
+    - the circle S1 and the sphere S2
+    - the hyperbolic plane (the Poincare disk, the Poincare
+      half plane and the Klein disk)
+    - the Poincare polydisk
+
+    Parameters
+    ----------
+    points : array-like, shape=[..., dim]
+        Points to be plotted.
+    space: str, optional, {'SO3_GROUP', 'SE3_GROUP', 'S1', 'S2',
+        'H2_poincare_disk', 'H2_poincare_half_plane', 'H2_klein_disk',
+        'poincare_polydisk'}
+    point_type: str, optional, {'extrinsic', 'ball', 'half-space'}
     """
     if space not in IMPLEMENTED:
         raise NotImplementedError(
@@ -536,12 +598,17 @@ def plot(points, ax=None, space=None,
         sphere.draw(ax, **point_draw_kwargs)
 
     elif space == 'H2_poincare_disk':
+        if point_type is None:
+            point_type = 'extrinsic'
         poincare_disk = PoincareDisk(point_type=point_type)
         ax = poincare_disk.set_ax(ax=ax)
         poincare_disk.add_points(points)
         poincare_disk.draw(ax, **point_draw_kwargs)
+        plt.axis('off')
 
     elif space == 'poincare_polydisk':
+        if point_type is None:
+            point_type = 'extrinsic'
         n_disks = points.shape[1]
         poincare_poly_disk = PoincarePolyDisk(point_type=point_type,
                                               n_disks=n_disks)
@@ -560,8 +627,10 @@ def plot(points, ax=None, space=None,
             poincare_poly_disk.draw(ax, **point_draw_kwargs)
 
     elif space == 'H2_poincare_half_plane':
-        poincare_half_plane = PoincareHalfPlane()
-        ax = poincare_half_plane.set_ax(ax=ax)
+        if point_type is None:
+            point_type = 'half-space'
+        poincare_half_plane = PoincareHalfPlane(point_type=point_type)
+        ax = poincare_half_plane.set_ax(points=points, ax=ax)
         poincare_half_plane.add_points(points)
         poincare_half_plane.draw(ax, **point_draw_kwargs)
 
@@ -571,4 +640,10 @@ def plot(points, ax=None, space=None,
         klein_disk.add_points(points)
         klein_disk.draw(ax, **point_draw_kwargs)
 
+    elif space == 'SE2_GROUP':
+        plane = SpecialEuclidean2()
+        ax = plane.set_ax(ax=ax)
+        plane.add_points(points)
+        plane.draw(ax, **point_draw_kwargs)
+
     return ax
